#ifndef __STRID_H__
#define __STRID_H__

#include <windows.h>

typedef  DWORD ID_TYPE;
#define  INVALID_ID   ((ID_TYPE)0xFFFFFFFF)

struct StringID
{
protected:
	char m_viewstr[5];
	char m_hexstr[20];
public:
	union	{ DWORD id; char str[5]; };
	
	StringID();	StringID(DWORD i); 	StringID(char *s);

	char * GetByString();
	char * GetByHex();

	BOOL IsValid();
	
	operator DWORD(void);
	operator char*(void);

	DWORD operator =(StringID &strid);
	DWORD operator =(DWORD id);
	DWORD operator =(char *s);

	BOOL operator ==(StringID &strid);
	BOOL operator !=(StringID &strid);
	BOOL operator >(StringID &strid);
	BOOL operator <(StringID &strid);
	BOOL operator >=(StringID &strid);
	BOOL operator <=(StringID &strid);

	BOOL operator ==(DWORD id);
	BOOL operator !=(DWORD id);
	BOOL operator >(DWORD id);
	BOOL operator <(DWORD id);
	BOOL operator >=(DWORD id);
	BOOL operator <=(DWORD id);

	BOOL operator ==(char *s);
	BOOL operator !=(char *s);
	BOOL operator >(char *s);
	BOOL operator <(char *s);
	BOOL operator >=(char *s);
	BOOL operator <=(char *s);

};

// Constructor
inline StringID::StringID()        { StringID::id = INVALID_ID; StringID::str[4] =0; }
inline StringID::StringID(DWORD i) { StringID::id = i; StringID::str[4] =0; }
inline StringID::StringID(char *s) { *this = s; StringID::str[4] =0; }

inline char * StringID::GetByString() 
{
	for(int i=0; i<4; i++)
	{
		m_viewstr[i] = str[3-i];
	}
	m_viewstr[4] = 0;
	return m_viewstr;
}

inline char * StringID::GetByHex()
{
	wsprintf(&m_hexstr[0], "%X", id); return &m_hexstr[0];
}

// Casting Operator
inline StringID::operator DWORD() { return StringID::id; }
inline StringID::operator char*() { return StringID::GetByString(); }

// Replacement Operator 
inline DWORD StringID::operator =(StringID &strid) { return StringID::id = strid.id; }
inline DWORD StringID::operator =(DWORD id)     { return StringID::id = id; }
inline DWORD StringID::operator =(char *s) 
{
	s = s+3;
	for(int i=0; i<sizeof(DWORD); i++)	{ StringID::str[i] = *s; s--; }
	StringID::str[4] = 0;
	return StringID::id; 
}

inline BOOL StringID::IsValid()
{
	for(int i=0; i<4; i++)
	{
		if(!( 
			((str[i]>='0')&&(str[i] <='9')) || 
			((str[i]>='A')&&(str[i] <='Z')) ||
			(str[i] == '_')
			))
			return FALSE;
	}
	return TRUE;
}

// Campare Operator
inline BOOL StringID::operator ==(StringID &strid) {	return (StringID::id == strid.id); }
inline BOOL StringID::operator !=(StringID &strid) {	return (StringID::id != strid.id); }
inline BOOL StringID::operator >(StringID &strid)  {	return (StringID::id > strid.id); }
inline BOOL StringID::operator <(StringID &strid)  {	return (StringID::id < strid.id); }
inline BOOL StringID::operator >=(StringID &strid) {	return (StringID::id >= strid.id); }
inline BOOL StringID::operator <=(StringID &strid) {	return (StringID::id <= strid.id); }

inline BOOL StringID::operator ==(DWORD id) {	return (StringID::id == id); }
inline BOOL StringID::operator !=(DWORD id) {	return (StringID::id != id); }
inline BOOL StringID::operator >(DWORD id)  {	return (StringID::id > id); }
inline BOOL StringID::operator <(DWORD id)  {	return (StringID::id < id); }
inline BOOL StringID::operator >=(DWORD id) {	return (StringID::id >= id); }
inline BOOL StringID::operator <=(DWORD id) {	return (StringID::id <= id); }

inline BOOL StringID::operator ==(char *s) {	return (StringID::id == StringID(s)); }
inline BOOL StringID::operator !=(char *s) {	return (StringID::id != StringID(s)); }
inline BOOL StringID::operator >(char *s)  {	return (StringID::id > StringID(s)); }
inline BOOL StringID::operator <(char *s)  {	return (StringID::id < StringID(s)); }
inline BOOL StringID::operator >=(char *s) {	return (StringID::id >= StringID(s)); }
inline BOOL StringID::operator <=(char *s) {	return (StringID::id <= StringID(s)); }

inline void MakeValidIDString(char * str)
{
	for(int i=0; i<4; i++)
	{
		if((str[i]>='a')&&(str[i] <='z'))
			str[i] = str[i]-('a'-'A');
	}
}

#define MAKE_ID(i1, i2, i3, i4) ((ID_TYPE)((i1<<24)|(i2<<16)|(i3<<8)|(i4)))
#define MAKE_ID2(i1, i2, i3, i4) ((ID_TYPE)((#@i1<<24)|(#@i2<<16)|(#@i3<<8)|(#@i4)))

#define Axxx ((DWORD)('A'<<24))
#define Bxxx ((DWORD)('B'<<24))
#define Cxxx ((DWORD)('C'<<24))
#define Dxxx ((DWORD)('D'<<24))
#define Exxx ((DWORD)('E'<<24))
#define Fxxx ((DWORD)('F'<<24))
#define Gxxx ((DWORD)('G'<<24))
#define Hxxx ((DWORD)('H'<<24))
#define Ixxx ((DWORD)('I'<<24))
#define Jxxx ((DWORD)('J'<<24))
#define Kxxx ((DWORD)('K'<<24))
#define Lxxx ((DWORD)('L'<<24))
#define Mxxx ((DWORD)('M'<<24))
#define Nxxx ((DWORD)('N'<<24))
#define Oxxx ((DWORD)('O'<<24))
#define Pxxx ((DWORD)('P'<<24))
#define Qxxx ((DWORD)('Q'<<24))
#define Rxxx ((DWORD)('R'<<24))
#define Sxxx ((DWORD)('S'<<24))
#define Txxx ((DWORD)('T'<<24))
#define Uxxx ((DWORD)('U'<<24))
#define Vxxx ((DWORD)('V'<<24))
#define Wxxx ((DWORD)('W'<<24))
#define Xxxx ((DWORD)('X'<<24))
#define Yxxx ((DWORD)('Y'<<24))
#define Zxxx ((DWORD)('Z'<<24))


#define xAxx ((DWORD)('A'<<16))
#define xBxx ((DWORD)('B'<<16))
#define xCxx ((DWORD)('C'<<16))
#define xDxx ((DWORD)('D'<<16))
#define xExx ((DWORD)('E'<<16))
#define xFxx ((DWORD)('F'<<16))
#define xGxx ((DWORD)('G'<<16))
#define xHxx ((DWORD)('H'<<16))
#define xIxx ((DWORD)('I'<<16))
#define xJxx ((DWORD)('J'<<16))
#define xKxx ((DWORD)('K'<<16))
#define xLxx ((DWORD)('L'<<16))
#define xMxx ((DWORD)('M'<<16))
#define xNxx ((DWORD)('N'<<16))
#define xOxx ((DWORD)('O'<<16))
#define xPxx ((DWORD)('P'<<16))
#define xQxx ((DWORD)('Q'<<16))
#define xRxx ((DWORD)('R'<<16))
#define xSxx ((DWORD)('S'<<16))
#define xTxx ((DWORD)('T'<<16))
#define xUxx ((DWORD)('U'<<16))
#define xVxx ((DWORD)('V'<<16))
#define xWxx ((DWORD)('W'<<16))
#define xXxx ((DWORD)('X'<<16))
#define xYxx ((DWORD)('Y'<<16))
#define xZxx ((DWORD)('Z'<<16))


#define xxAx ((DWORD)('A'<<8))
#define xxBx ((DWORD)('B'<<8))
#define xxCx ((DWORD)('C'<<8))
#define xxDx ((DWORD)('D'<<8))
#define xxEx ((DWORD)('E'<<8))
#define xxFx ((DWORD)('F'<<8))
#define xxGx ((DWORD)('G'<<8))
#define xxHx ((DWORD)('H'<<8))
#define xxIx ((DWORD)('I'<<8))
#define xxJx ((DWORD)('J'<<8))
#define xxKx ((DWORD)('K'<<8))
#define xxLx ((DWORD)('L'<<8))
#define xxMx ((DWORD)('M'<<8))
#define xxNx ((DWORD)('N'<<8))
#define xxOx ((DWORD)('O'<<8))
#define xxPx ((DWORD)('P'<<8))
#define xxQx ((DWORD)('Q'<<8))
#define xxRx ((DWORD)('R'<<8))
#define xxSx ((DWORD)('S'<<8))
#define xxTx ((DWORD)('T'<<8))
#define xxUx ((DWORD)('U'<<8))
#define xxVx ((DWORD)('V'<<8))
#define xxWx ((DWORD)('W'<<8))
#define xxXx ((DWORD)('X'<<8))
#define xxYx ((DWORD)('Y'<<8))
#define xxZx ((DWORD)('Z'<<8))

#define _0xxx ((DWORD)('0'<<24))
#define _1xxx ((DWORD)('1'<<24))
#define _2xxx ((DWORD)('2'<<24))
#define _3xxx ((DWORD)('3'<<24))
#define _4xxx ((DWORD)('4'<<24))
#define _5xxx ((DWORD)('5'<<24))
#define _6xxx ((DWORD)('6'<<24))
#define _7xxx ((DWORD)('7'<<24))
#define _8xxx ((DWORD)('8'<<24))
#define _9xxx ((DWORD)('9'<<24))

#define _x0xx ((DWORD)('0'<<16))
#define _x1xx ((DWORD)('1'<<16))
#define _x2xx ((DWORD)('2'<<16))
#define _x3xx ((DWORD)('3'<<16))
#define _x4xx ((DWORD)('4'<<16))
#define _x5xx ((DWORD)('5'<<16))
#define _x6xx ((DWORD)('6'<<16))
#define _x7xx ((DWORD)('7'<<16))
#define _x8xx ((DWORD)('8'<<16))
#define _x9xx ((DWORD)('9'<<16))

#define _xx0x ((DWORD)('0'<<8))
#define _xx1x ((DWORD)('1'<<8))
#define _xx2x ((DWORD)('2'<<8))
#define _xx3x ((DWORD)('3'<<8))
#define _xx4x ((DWORD)('4'<<8))
#define _xx5x ((DWORD)('5'<<8))
#define _xx6x ((DWORD)('6'<<8))
#define _xx7x ((DWORD)('7'<<8))
#define _xx8x ((DWORD)('8'<<8))
#define _xx9x ((DWORD)('9'<<8))

#define _xxx0 ((DWORD)('0'))
#define _xxx1 ((DWORD)('1'))
#define _xxx2 ((DWORD)('2'))
#define _xxx3 ((DWORD)('3'))
#define _xxx4 ((DWORD)('4'))
#define _xxx5 ((DWORD)('5'))
#define _xxx6 ((DWORD)('6'))
#define _xxx7 ((DWORD)('7'))
#define _xxx8 ((DWORD)('8'))
#define _xxx9 ((DWORD)('9'))

#define _Xxxx ((DWORD)(0xff<<24))
#define _xXxx ((DWORD)(0xff<<16))
#define _xxXx ((DWORD)(0xff<<8))
#define _xxxX ((DWORD)(0xff))

#endif






