// TimeCheck.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <mmsystem.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>

//! INTORFLOAT union for easy access to bits of a float.
typedef union
{
    int     i;          // as integer
    float   f;          // as float
    struct              // as bit fields
    {
        unsigned int    sign:1;
        unsigned int    biasedexponent:8;
        unsigned int    significand;
    } bits;
} INTORFLOAT;


INTORFLOAT  FTOIBIAS = {((23 - 0 + 127) << 23) + (1 << 22)};

inline int f2i(float f) 
{
	INTORFLOAT ftmp;
	ftmp.f = f;
	ftmp.f += FTOIBIAS.f;
	ftmp.i -= FTOIBIAS.i;
	return ftmp.i;	
}


inline int fastf2i_round(float f)
{
#ifdef _WIN32
	int i;
	__asm
	{
		fld		f
		fistp	i
	}
	return i;
#else
	return (int)(f + 0.5f);
#endif	
}


inline int fastf2i(float fValue)
{
#ifdef _WIN32
	static const float round_to_nearest = -0.499999f;

    int iValue;
    __asm 
	{
		fld		fValue
		fadd	round_to_nearest
		fistp	iValue
	}
    return iValue;
#else
    return (int)fValue;
#endif
}

inline int Float2Int( float a )
{
#ifdef _WIN32
	int CtrlwdHolder;
	int CtrlwdSetter;
	int RetVal;
	__asm 
	{
		fld    a					// push 'a' onto the FP stack
		fnstcw CtrlwdHolder			// store FPU control word
		movzx  eax, CtrlwdHolder	// move and zero extend word into eax
		and    eax, 0xFFFFF3FF		// set all bits except rounding bits to 1
		or     eax, 0x00000C00		// set rounding mode bits to round down
		mov    CtrlwdSetter, eax	// Prepare to set the rounding mode -- prepare to enter plaid!
		fldcw  CtrlwdSetter			// Entering plaid!
		fistp  RetVal				// Store and converted (to int) result
		fldcw  CtrlwdHolder			// Restore control word
	}
	return RetVal;
#else
	return (int)a;
#endif	
}


inline void Float2IntModeChopping()
{
#ifdef _WIN32
	int CtrlwdHolder;
	int CtrlwdSetter;
	__asm 
	{
		fnstcw CtrlwdHolder			// store FPU control word
		movzx  eax, CtrlwdHolder	// move and zero extend word into eax
		and    eax, 0xFFFFF3FF		// set all bits except rounding bits to 1
		or     eax, 0x00000C00		// set rounding mode bits to round down
		mov    CtrlwdSetter, eax	// Prepare to set the rounding mode -- prepare to enter plaid!
		fldcw  CtrlwdSetter			// Entering plaid!
	}
#endif
}

void test4() 
{
	int i;
	DWORD stime1, stime2, stime3, stime4, stime5;
	DWORD result1 = 0, result2 = 0, result3 = 0, result4 = 0, result5 = 0;

	int a = 0;
	int b = 0;
	int c = 0;
	int d = 0;
	int e = 0;

	float v = 1234.56f;


	stime1 = timeGetTime();
	while(stime1 == timeGetTime());
	for(i = 0; i < 5000000; i++) {
		a = fastf2i_round(v);

	}
	for(i = 0; i < 5000000; i++) {
		a = fastf2i_round(v);
	}

	result1 = timeGetTime() - stime1;

	stime2 = timeGetTime();
	while(stime2 == timeGetTime());
	for(i = 0; i < 5000000; i++) {
		b = fastf2i(v);
	}
	for(i = 0; i < 5000000; i++) {
		b = fastf2i(v);
	}
	result2 = timeGetTime() - stime2;

	stime3 = timeGetTime();
	while(stime3 == timeGetTime());
	for(i = 0; i < 5000000; i++) {
		c = Float2Int(v);
	}
	for(i = 0; i < 5000000; i++) {
		c = Float2Int(v);
	}
	result3 = timeGetTime() - stime3;

	Float2IntModeChopping();

	stime4 = timeGetTime();
	while(stime4 == timeGetTime());
	for(i = 0; i < 5000000; i++) {
		d = f2i(v);
	}
	for(i = 0; i < 5000000; i++) {
		d = f2i(v);
	}
	result4 = timeGetTime() - stime4;

	stime5 = timeGetTime();
	while(stime5 == timeGetTime());
	for(i = 0; i < 5000000; i++) {
		e = int(v);
	}
	for(i = 0; i < 5000000; i++) {
		e = int(v);
	}
	result5 = timeGetTime() - stime5;


	printf("־  : %5.3f\n", v);
	printf("fastf2i_round Time : %d (%d)\n", result1, a);
	printf("fastf2i       Time : %d (%d)\n", result2, b);
	printf("Float2Int     Time : %d (%d)\n", result3, c);
	printf("f2i           Time : %d (%d)\n", result4, d);
	printf("int           Time : %d (%d)\n", result5, e);

}



int __cdecl main(int argc, char* argv[])
{
	test4();
	
	return 0;
}
